import { Injectable, CanActivate, ExecutionContext, UnauthorizedException } from '@nestjs/common';
import { Reflector } from '@nestjs/core';
import { Observable, of } from 'rxjs';
import { SessionsService } from '../services/sessions.service';

export class PermissionGuardArgs {
    keywords?: string[] | null | undefined;
    verificationLevels?: string[] | null | undefined;
}

export const AllowPermissions = Reflector.createDecorator<PermissionGuardArgs>();

@Injectable()
export class PermissionGuard implements CanActivate {
    constructor(private readonly _sessionsService: SessionsService, private readonly _reflector: Reflector) {}

    canActivate(context: ExecutionContext): Observable<boolean> {
        const args = this._reflector.get(PermissionGuardArgs, context.getHandler());
        const request = context.switchToHttp().getRequest();
        const token: string = this._getToken(request);

        if (!token) {
            throw new UnauthorizedException();
        }

        console.log(args);

        return of(true);
    }

    private _getToken(request: any) {
        let token = request?.headers?.authorization;

        if (!token) {
            return null;
        }

        token = token.replace('bearer ', '');
        token = token.replace('Bearer ', '');

        return token;
    }

    /*
    private _checkPermission(userPermissions: IPermission[], allowedPermission: string): boolean {
        if (allowedPermission) {
            return userPermissions.some((item) => item.keyword === allowedPermission);
        } else {
            return true;
        }
    }

    private setUserData(userData: ISessionUserData, client: IClient, permissions: IPermission[], request: Request) {
        const userInfo = {
            ...userData,
            client,
            permissions,
        };
        request['user'] = userInfo;
    }

    private validateUserData(userData: ISessionUserData) {
        if (!userData) {
            throw new RESTError('unauthorized', HttpStatus.UNAUTHORIZED);
        }
        return userData;
    }
    */
}
